#pragma once

class State_machine::Condition
{
	private:
		State_machine::Env& _env;
		Genode::Xml_node const _config;
		bool _result { false };
		Genode::Signal_context_capability _sigh { };

	protected:
		State_machine::Env& env() { return _env; }
		Genode::Xml_node const& config() { return _config; }
		virtual bool evaluate() = 0;

		void notify_sigh()
		{
			if (_sigh.valid())
				Genode::Signal_transmitter(_sigh).submit();
		}

	public:
		bool result() { return _result; }

		void compute()
		{
			_result = evaluate();
		}

		void sigh(Genode::Signal_context_capability sigh)
		{
			_sigh = sigh;
		}

		Condition(State_machine::Env& env, Genode::Xml_node const& config)
		 : _env(env), _config(config)
		{
		}

		virtual ~Condition()
		{
		}
};

class State_machine::Condition_factory
{
	public:
		virtual State_machine::Condition& create(State_machine::Env& env, Genode::Xml_node const& config) = 0;

		virtual ~Condition_factory()
		{
		}
};

